package com.loafle.overflow.container.general.crawler.impl.wmi;

import com.google.gson.Gson;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.model.sensorconfig.ResultSet;
import com.loafle.overflow.model.sensorconfig.SensorConfig;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class WMICrawler implements Crawler {
  private Gson gson;

  public WMICrawler() {
    this.gson = new Gson();
  }

  @Override
  public String name() {
    return "WMI";
  }

  @Override
  public String toString() {
    return "WMI Crawler";
  }

  @Override
  public void auth(Map<String, Object> authIMap) throws OverflowException {
    throw new OverflowException("Not implemented", null);
  }

  @Override
  public Map<String, String> get(SensorConfig sensorConfig) throws OverflowException {
    try {
      return WMICrawlerOS.getInstance().process(sensorConfig);
    } catch (Exception e) {
      throw new OverflowException("WMICrawler", e);
    }
  }

  public Object getMeta(SensorConfig sensorConfig) throws OverflowException {

    try {
      List<SensorConfig> metaConfigList = loadMetaConfig();

      WMICrawlerOS os = WMICrawlerOS.getInstance();
      Map<String, String> returnMap = new HashMap<String, String>();
      ResultSet ofResultSet = null;
      for (SensorConfig metaConfig : metaConfigList) {
        for (int indexI = 0; indexI < metaConfig.getItems().size(); ++indexI) {
          ofResultSet = os.processWMI(sensorConfig, metaConfig.getItems().get(indexI));
          if (ofResultSet != null) {
            returnMap.putAll(ofResultSet.getData());
          }
        }
      }        
    } catch (Exception e) {
      throw new OverflowException("WMICrawler", e);
    }

    return null;
  }

  private List<SensorConfig> loadMetaConfig() throws IOException {
    ClassLoader classLoader = getClass().getClassLoader();

    URL url = classLoader.getResource("config/wmi/meta");
    if (url == null) {
      return null;
    }

    String path = url.getFile();
    File pathFile = new File(path);

    System.out.println(path);
    String[] metaFiles = pathFile.list();
    List<SensorConfig> retList = new ArrayList<>();
    SensorConfig metaConfig = null;
    for (String fi : metaFiles) {
      if (fi.indexOf("meta_") > -1) {
        metaConfig = new Gson().fromJson(new InputStreamReader(new FileInputStream(new File(path + "/" + fi))), SensorConfig.class);
        if (metaConfig != null) {
          retList.add(metaConfig);
        }
      }
    }

    return retList;
  }

}
